﻿using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 다각선
        /// </summary>
        private Polyline polyline;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.polyline = new Polyline();

            this.polyline.Stroke = Brushes.Blue;

            this.canvas.Children.Add(polyline);

            this.canvas.SizeChanged += canvas_SizeChanged;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 마우스 DOWN 처리하기 - OnMouseDown(e)

        /// <summary>
        /// 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            base.OnMouseDown(e);

            Point mousePoint = e.GetPosition(this.canvas);

            if(e.ChangedButton == MouseButton.Left)
            {
                this.control1Point.Center = mousePoint;
            }

            if(e.ChangedButton == MouseButton.Right)
            {
                this.control2Point.Center = mousePoint;
            }

            DrawBezierCurve();
        }

        #endregion
        #region 마우스 이동시 처리하기 - OnMouseMove(e)

        /// <summary>
        /// 마우스 이동시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            Point mousePoint = e.GetPosition(this.canvas);

            if(e.LeftButton == MouseButtonState.Pressed)
            {
                this.control1Point.Center = mousePoint;
            }

            if(e.RightButton == MouseButtonState.Pressed)
            {
                this.control2Point.Center = mousePoint;
            }

            DrawBezierCurve();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 캔버스 크기 변경시 처리하기 - canvas_SizeChanged(sender, e)

        /// <summary>
        /// 캔버스 크기 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvas_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            this.startPoint.Center    = new Point(    e.NewSize.Width / 4,     e.NewSize.Height / 2);
            this.control1Point.Center = new Point(    e.NewSize.Width / 2,     e.NewSize.Height / 4);
            this.control2Point.Center = new Point(    e.NewSize.Width / 2, 3 * e.NewSize.Height / 4);
            this.endPoint.Center      = new Point(3 * e.NewSize.Width / 4,     e.NewSize.Height / 2);

            DrawBezierCurve();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 베지어 곡선 그리기 - DrawBezierCurve()

        /// <summary>
        /// 베지어 곡선 그리기
        /// </summary>
        private void DrawBezierCurve()
        {
            Point[] pointArray = new Point[10];

            for(int i = 0; i < pointArray.Length; i++)
            {
                double t = (double)i / (pointArray.Length - 1);

                double x = (1 - t) * (1 - t) * (1 - t) * this.startPoint.Center.X +
                           3 * t * (1 - t) * (1 - t) * this.control1Point.Center.X +
                           3 * t * t * (1 - t) * this.control2Point.Center.X +
                           t * t * t * this.endPoint.Center.X;

                double y = (1 - t) * (1 - t) * (1 - t) * this.startPoint.Center.Y +
                           3 * t * (1 - t) * (1 - t) * this.control1Point.Center.Y +
                           3 * t * t * (1 - t) * this.control2Point.Center.Y +
                           t * t * t * this.endPoint.Center.Y;

                pointArray[i] = new Point(x, y);
            }

            this.polyline.Points = new PointCollection(pointArray);
        }

        #endregion
    }
}